#!/usr/bin/env python3

# -*- encoding: utf-8; py-indent-offset: 4 -*-

# License: GNU General Public License v2

from typing import Dict
import datetime
import pytz
from dateutil import tz
import re
from cmk.agent_based.v2 import (
    AgentSection,
    CheckPlugin,
    CheckResult,
    DiscoveryResult,
    Result,
    Service,
    State,
    Metric,
)

non_graphable = [
    "eDirectorySystemCurrTime",
    "eDirectoryUpTime",
    "eDirectoryAgentVersion",
    "CheckPointThreadStartTime",
    "CheckPointThreadForceStartTime",
    "CheckPointThreadIsForced",
]

uptime_attributes = [
    "eDirectoryUpTime",
]

time_attributes = [
    "eDirectorySystemCurrTime",
    "CheckPointThreadStartTime",
]

def format_partition_agent(value):
    formatted = re.sub('CN=|OU=|O=|T=', '', value).replace("=", " ")
    return formatted

def convert_timestamp(value):
    """convert Zulu time to current time in local timezone"""
    utc_dt = datetime.datetime.strptime(str(value), "%Y%m%d%H%M%SZ")
    utc = pytz.utc
    utc_dt = utc_dt.replace(tzinfo=tz.UTC)
    local_tz = tz.tzlocal()
    local_dt = utc_dt.astimezone(local_tz)
    return local_dt

def parse_ldap_data(string_table):
    """parse one lines of data to dictionary"""
    parsed = {}
    for line in string_table:
        item = line[0]
        parsed.setdefault(item, {})
        for _count, data in enumerate(line[1:]):
            if item == "Agent Partition":
                #print(format_partition_agent(data))
                continue
                #key, value=format_partition_agent(data).split("#")
            else:
                key, value=data.split("=")
            parsed[item].setdefault(key, value)
    return parsed

def discover_edirectory_items(section) -> DiscoveryResult:
    '''discover one item per key'''
    for key, data in section.items():
        if key == "Agent Partition":
            continue
        yield Service(item=key)

def check_edirectory_items(item: str, section) -> CheckResult:
    data = section.get(item)
    if not data:
        return
    for key, value in data.items():
        if key in uptime_attributes:
            uptime = datetime.timedelta(seconds=float(value))
            yield Result(state=State(0), summary=f"{key}: {uptime}", details=f"{key}: {uptime}")
        if key in time_attributes:
            datevalue = convert_timestamp(value)
            yield Result(state=State(0), summary=f"{key}: {datevalue}", details=f"{key}: {datevalue}")
        else:
            yield Result(state=State(0), summary=f"{key}: {value}", details=f"{key}: {value}")
        if key not in non_graphable:
            yield Metric(key, float(value))

agent_section_edirectory_monitor = AgentSection(
    name="edirectory_monitor",
    parse_function=parse_ldap_data,
    )

check_plugin_edirectory_monitor = CheckPlugin(
    name="edirectory_monitor",
    service_name="%s",
    discovery_function=discover_edirectory_items,
    check_function=check_edirectory_items,
)
